/*
    SPDX-FileCopyrightText: none

    SPDX-License-Identifier: GPL-2.0-only
*/
#define HAVE_TEXT_AUTOGENERATE_TEXT 1
#define HAVE_KTEXTADDONS_TEXT_TO_SPEECH_SUPPORT 1
#define HAVE_TEXTAUTOGENERATE_INTERNAL_TOOLS 0
